import { HapClient, HapMonitor } from "../HapClient";
import { log } from '../Types/types';
import { singleton, inject, container } from 'tsyringe';
import { IServiceType } from "../HapClient/interfaces";
import { IConfig } from "../Models/IConfig";
import { Database } from "sqlite3";
import { mkdirSync } from 'fs';
import { join } from 'path';
import Knex = require("knex");


@singleton()
export default class Monitor {
    private log: log;
    private config: IConfig
    private client: HapClient;
    private pin = "031-45-154";
    private databaseLocation = `/Users/brandonwatson/.homebridge/automation`;
    private databaseName = 'automation.db';
    private db?: Database;
    private knex?: Knex;

    constructor(@inject("log") log: log, @inject("config") config: IConfig) {
        this.log = log;
        this.config = config;

        this.client = new HapClient({
            pin: this.pin,
            logger: this.log,
            config: {}
        });

        //Overwrite default db location
        if (config.databaseLocation) {
            this.databaseLocation = config.databaseLocation;
        }

        this.initializeDatabase().catch((err) => {
            this.log(`Error connecting to sqlite database: ${err}`);
        });

        this.initializeHapMonitor();
    }

    private get dbFullPath(): string {
        return join(this.databaseLocation, this.databaseName);
    }

    private initializeHapMonitor = async (): Promise<void> => {
        await this.client.discover();
        const services: Array<IServiceType> = await this.client.getAllServices();

        const hapMonitor = new HapMonitor(this.log, this.log, this.pin, services);

        //Set service updated event
        hapMonitor.on('service-updated', this.onServiceUpdated);

        this.log("Monitor ready");
    }

    private initializeDatabase = async (): Promise<void> => {
        try {
            mkdirSync(this.databaseLocation);
        } catch (err) {
            if (err.code !== 'EEXIST') {
                throw err;
            }
        }

        try {
            await this.initDbFile(this.dbFullPath);
        } catch (err) {
            throw err;
        }

        this.knex = Knex({
            client: 'sqlite3',
            connection: {
                filename: this.dbFullPath
            }
        });

        container.register<Knex>('knex', { useValue: this.knex });

        const hasTables = await this.knex.schema.hasTable('events');

        if (!hasTables) {
            //create databse tables
            await this.knex.raw(`CREATE TABLE "event_service_instance_characteristics" (
            "id"	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE,
            "instance_id"	INTEGER NOT NULL,
            "key"	TEXT NOT NULL,
            "type"	TEXT NOT NULL,
            "value"	BLOB NOT NULL)`);

            await this.knex.raw(`CREATE TABLE "event_service_instances" (
            "id"	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE,
            "event_id"	INTEGER NOT NULL,
            "service_id"	INTEGER NOT NULL)`);

            await this.knex.raw(`CREATE TABLE "events" (
            "id"	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE,
            "timestamp"	BLOB NOT NULL)`);

            await this.knex.raw(`CREATE TABLE "services" (
            "id"	INTEGER NOT NULL UNIQUE,
            "uniqueId"	TEXT NOT NULL UNIQUE,
            "uuid"	TEXT NOT NULL UNIQUE,
            "type"	TEXT NOT NULL,
            "humanType"	TEXT NOT NULL,
            "serviceName"	TEXT NOT NULL)`);
        }
        console.log();
    };

    private initDbFile = (path: string): Promise<Database> => {
        return new Promise((resolve, reject): void => {
            const db: Database = new Database(path, (err): void => {
                if (err) {
                    return reject(err);
                } else {
                    return resolve(db);
                }
            });
        });
    }

    private syncDatabase = async (): Promise<void> => {

    }

    private onServiceUpdated = (updatedServices: Array<IServiceType>): void => {
        console.log("asdf");

    }
}